<?php

/*
 * this class should be used to stores properties and methods shared by the
 * admin and public side of wordpress
 */

class Dauc_Shared {

	//regex
    public $regex_capability = '/^\s*[A-Za-z0-9_]+\s*$/';
	public $regex_list_of_post_types = '/^(\s*([A-Za-z0-9_-]+\s*,\s*)+[A-Za-z0-9_-]+\s*|\s*[A-Za-z0-9_-]+\s*)$/';
	public $regex_borderDash = '/^(\s*([0-9]+\s*,\s*)+[0-9]+\s*|\s*[0-9]+\s*)$/';
    public $digits_regex = '/^\s*\d+\s*$/';

	protected static $instance = null;

	private $data = array();

	private function __construct() {

		//Set plugin textdomain
		load_plugin_textdomain( 'dauc', false, 'uberchart/lang/' );

		$this->data['slug'] = 'dauc';
		$this->data['ver']  = '1.34';
		$this->data['dir']  = substr( plugin_dir_path( __FILE__ ), 0, - 7 );
		$this->data['url']  = substr( plugin_dir_url( __FILE__ ), 0, - 7 );

	}

	public static function get_instance() {

		if ( null == self::$instance ) {
			self::$instance = new self;
		}

		return self::$instance;

	}

	//retrieve data
	public function get( $index ) {
		return $this->data[ $index ];
	}

	/*
	 * Insert the default data for a single data structure of a row of a chart
	 *
	 * @param $chart_id The id of the chart
	 * @param $row_index The index of the data structure row
	 * @param $row_data_json The data of a single data structure row in the json format
	 */
	public function data_insert_default_record($chart_id, $row_index, $row_data_json){

		//save in the db table
		global $wpdb;
		$table_name = $wpdb->prefix . $this->get( 'slug' ) . "_data";
		$safe_sql   = $wpdb->prepare( "INSERT INTO $table_name SET
            chart_id = %d,
            row_index = %d,
            content = %s,
            label = %s,
            fill = %d,
            lineTension = %f,
            backgroundColor = %s,
            borderWidth = %d,
            borderColor = %s,
            borderCapStyle = %s,
            borderDash = %s,
            borderDashOffset = %f,
            borderJoinStyle = %s,
            pointBorderColor = %s,
            pointBackgroundColor = %s,
            pointBorderWidth = %d,
            pointRadius = %d,
            pointHoverRadius = %d,
            pointHitRadius = %d,
            pointHoverBackgroundColor = %s,
            pointHoverBorderColor = %s,
            pointHoverBorderWidth = %d,
            pointStyle = %s,
            showLine = %d,
            spanGaps = %d,
            hoverBackgroundColor = %s,
            hoverBorderColor = %s,
            hoverBorderWidth = %d,
            hitRadius = %d,
            hoverRadius = %d,
            plotY2 = %d,
            hidden = %d",
			$chart_id,
			$row_index,
			$row_data_json,
			'Label ' . $row_index,
			false,
			0,
			'rgba(0,0,0,0.1)',
			3,
			'rgba(0,0,0,0.1)',
			'butt',
			0,
			0.0,
			'miter',
			'rgba(0,0,0,0.1)',
			'rgba(0,0,0,0.1)',
			1,
			1,
			1,
			5,
			'rgba(0,0,0,0.1)',
			'rgba(0,0,0,0.1)',
			2,
			'circle',
			true,
			false,
			'rgba(0,0,0,0.1)',
			'rgba(0,0,0,0.1)',
			1,
			1,
			1,
			0,
            0
		);

		$query_result = $wpdb->query( $safe_sql );

	}

	/*
	 * Given a string this method returns and escaped version suitable to be used in javascript strings that make use
	 * of a single quote as the delimiter of the string.
	 *
	 * @param string
	 * @return string The escaped version
	 */
	public function prepare_javascript_string($string) {

		$all = array(

			"\\" => "\\\\",
			"'" => "\\'"

		);

		$string = str_replace( array_keys( $all ), $all, $string );

		return $string;

	}

	/*
	 * Applies stripslashes to all the properties of an object
	 */
	public function object_stripslashes($obj){

		$property_a = get_object_vars($obj);

		foreach($property_a as $key => $value){

			$obj->{$key} = stripslashes($value);

		}

		return $obj;

	}

	/**
	 * Apply stripslashes on all the text fields of the chart.
	 *
	 * @param $chart_obj
	 *
	 * @return mixed
	 */
	function stripslashes_chart_obj($chart_obj){

		//The text field of the database. These fields require the application of stripslashes()
		$text_field_a = [
			'type',
			'name',
			'description',
			'labels',
			'canvas_backgroundColor',
			'title_position',
			'title_fontFamily',
			'title_fontColor',
			'title_fontStyle',
			'legend_position',
			'legend_labels_fontStyle',
			'legend_labels_fontColor',
			'legend_labels_fontFamily',
			'tooltips_mode',
			'tooltips_backgroundColor',
			'tooltips_titleFontFamily',
			'tooltips_titleFontStyle',
			'tooltips_titleFontColor',
			'tooltips_bodyFontFamily',
			'tooltips_bodyFontStyle',
			'tooltips_bodyFontColor',
			'tooltips_footerFontFamily',
			'tooltips_footerFontStyle',
			'tooltips_footerFontColor',
			'tooltips_multiKeyBackground',
			'tooltips_beforeTitle',
			'tooltips_afterTitle',
			'tooltips_beforeBody',
			'tooltips_afterBody',
			'tooltips_beforeLabel',
			'tooltips_afterLabel',
			'tooltips_beforeValue',
			'tooltips_afterValue',
			'tooltips_beforeFooter',
			'tooltips_footer',
			'tooltips_afterFooter',
			'animation_easing',
			'elements_rectangle_borderSkipped',
			'thousands_separator',
			'decimal_separator',
			'scales_xAxes_type',
			'scales_xAxes_position',
			'scales_xAxes_gridLines_color',
			'scales_xAxes_gridLines_lineWidth',
			'scales_xAxes_gridLines_zeroLineColor',
			'scales_xAxes_scaleLabel_labelString',
			'scales_xAxes_scaleLabel_fontColor',
			'scales_xAxes_scaleLabel_fontFamily',
			'scales_xAxes_scaleLabel_fontStyle',
			'scales_xAxes_ticks_fontColor',
			'scales_xAxes_ticks_fontFamily',
			'scales_xAxes_ticks_fontStyle',
			'scales_xAxes_ticks_prefix',
			'scales_xAxes_ticks_suffix',
			'scales_xAxes_ticks_min',
			'scales_xAxes_ticks_max',
			'scales_xAxes_ticks_maxTicksLimit',
			'scales_xAxes_ticks_stepSize',
			'scales_xAxes_ticks_suggestedMax',
			'scales_xAxes_ticks_suggestedMin',
			'scales_xAxes_time_format',
			'scales_xAxes_time_tooltipFormat',
			'scales_xAxes_time_unit_format',
			'scales_xAxes_time_unit',
			'scales_xAxes_time_max',
			'scales_xAxes_time_min',
			'scales_yAxes_type',
			'scales_yAxes_position',
			'scales_yAxes_gridLines_color',
			'scales_yAxes_gridLines_lineWidth',
			'scales_yAxes_gridLines_zeroLineColor',
			'scales_yAxes_scaleLabel_labelString',
			'scales_yAxes_scaleLabel_fontColor',
			'scales_yAxes_scaleLabel_fontFamily',
			'scales_yAxes_scaleLabel_fontStyle',
			'scales_yAxes_ticks_fontColor',
			'scales_yAxes_ticks_fontFamily',
			'scales_yAxes_ticks_fontStyle',
			'scales_yAxes_ticks_prefix',
			'scales_yAxes_ticks_suffix',
			'scales_yAxes_ticks_min',
			'scales_yAxes_ticks_max',
			'scales_yAxes_ticks_maxTicksLimit',
			'scales_yAxes_ticks_stepSize',
			'scales_yAxes_ticks_suggestedMax',
			'scales_yAxes_ticks_suggestedMin',
			'scales_y2Axes_type',
			'scales_y2Axes_position',
			'scales_y2Axes_gridLines_color',
			'scales_y2Axes_gridLines_lineWidth',
			'scales_y2Axes_gridLines_zeroLineColor',
			'scales_y2Axes_scaleLabel_labelString',
			'scales_y2Axes_scaleLabel_fontColor',
			'scales_y2Axes_scaleLabel_fontFamily',
			'scales_y2Axes_scaleLabel_fontStyle',
			'scales_y2Axes_ticks_fontColor',
			'scales_y2Axes_ticks_fontFamily',
			'scales_y2Axes_ticks_fontStyle',
			'scales_y2Axes_ticks_prefix',
			'scales_y2Axes_ticks_suffix',
			'scales_y2Axes_ticks_min',
			'scales_y2Axes_ticks_max',
			'scales_y2Axes_ticks_maxTicksLimit',
			'scales_y2Axes_ticks_stepSize',
			'scales_y2Axes_ticks_suggestedMax',
			'scales_y2Axes_ticks_suggestedMin',
			'scales_rl_gridLines_color',
			'scales_rl_gridLines_lineWidth',
			'scales_rl_angleLines_color',
			'scales_rl_angleLines_lineWidth',
			'scales_rl_pointLabels_fontColor',
			'scales_rl_pointLabels_fontFamily',
			'scales_rl_pointLabels_fontStyle',
			'scales_rl_ticks_prefix',
			'scales_rl_ticks_suffix',
			'scales_rl_ticks_fontColor',
			'scales_rl_ticks_fontFamily',
			'scales_rl_ticks_fontStyle',
			'scales_rl_ticks_min',
			'scales_rl_ticks_max',
			'scales_rl_ticks_suggestedMin',
			'scales_rl_ticks_suggestedMax',
			'scales_rl_ticks_stepSize',
			'scales_rl_ticks_maxTicksLimit',
			'scales_rl_ticks_backdropColor'
		];

		foreach($chart_obj as $key => $field){

			if(in_array($key, $text_field_a)){
				$chart_obj->$key = stripslashes($field);
			}

		}


		return $chart_obj;

	}

	/**
	 * Apply stripslashes on all the text fields of the data database.
	 *
	 * @param $chart_obj
	 *
	 * @return mixed
	 */
	function stripslashes_data($data_a){

		//The text field of the database. These fields require the application of stripslashes()
		$text_field_a = [
			'content',
			'label',
			'backgroundColor',
			'borderWidth',
			'borderColor',
			'borderCapStyle',
			'borderDash',
			'borderJoinStyle',
			'pointBorderColor',
			'pointBackgroundColor',
			'pointBorderWidth',
			'pointRadius',
			'pointHoverRadius',
			'pointHitRadius',
			'pointHoverBackgroundColor',
			'pointHoverBorderColor',
			'pointHoverBorderWidth',
			'pointStyle',
			'hoverBackgroundColor',
			'hoverBorderColor',
			'hoverBorderWidth',
			'hitRadius',
			'hoverRadius'
		];

		foreach($data_a as $key1 => $data){

			foreach($data as $key2 => $field){

				if(in_array($key2, $text_field_a)){
					$data_a[$key1][$key2] = stripslashes($field);
				}

			}

		}

		return $data_a;

	}

    /**
     * Apply the source data defined in the "Source" section.
     *
     * @param $chart
     * @return false|void
     */
    public function apply_source_data($chart){

		$modified_chart = $chart;
		
        /**
         * If an external source is used replace part of the data in $chart with the data available in the
         * external resource.
         *
         * - 0 = None
         * - 1 = JSON
         */
        if($chart->source_type === '1'){

            //Make the request
            $request = wp_remote_get( $chart->source_url);

            //If the remote request fails, wp_remote_get() will return a WP_Error. In case return the original chart.
            if( is_wp_error( $request ) ) {
                return $chart;
            }

			try{

				// Retrieve the data
				$body = wp_remote_retrieve_body( $request );
				$body_json = json_decode( $body );

				if($body_json !== null){

					//Replace the $modified_chart labels with the retrieved data
					if(isset($body_json->labels)){
						$modified_chart->labels = json_encode($body_json->labels);
					}

					//Iterate the chart data
					if(isset($body_json->data) and is_array($body_json->data)){

						foreach($body_json->data as $key => $dataset){

							if(isset($dataset->content)){
								$modified_chart->data[$key]['content'] = json_encode($dataset->content);
							}

							//Dataset properties
							if(isset($dataset->label)){
								$modified_chart->data[$key]['label'] = $dataset->label;
							}

						}

					}

				}

			}catch(Exception $e){

				return $chart;

			}

        }

        return $modified_chart;

    }

    /**
     * Set a transient based on the provided parameters and the "Transient Expiration" plugin option. Do not set a transient
     * if the "Transient Expiration" plugin option is equal to 0.
     *
     * @param $transient_name The name of the transient
     * @param $data The data of the transient
     */
    public function set_transient_based_on_settings($transient_name, $data){

        //Set the transient
        if ( $data->transient_expiration > 0 ) {
            set_transient( $transient_name, $data, $data->transient_expiration );
        }

    }

}